package nu.zoom.dsl.maven;

import nu.zoom.dsl.run.Runner;
import nu.zoom.dsl.run.ValidationException;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;

import java.io.File;
import java.util.Optional;

@Mojo(
        name = "endgen",
        defaultPhase = LifecyclePhase.GENERATE_SOURCES
)
public class EndgenMojo extends AbstractMojo {
    @Parameter(
            name = "templates",
            defaultValue = "${project.build.sourceDirectory}/main/endgen-templates"
    )
    File templates;

    @Parameter(
            name = "output",
            defaultValue = "${project.build.directory}/generated-sources/endgen"
    )
    File output;

    @Parameter(name = "dsl", required = true)
    File dsl;

    @Parameter(name = "parser")
    String parser;


    @Override
    public void execute() throws MojoExecutionException, MojoFailureException {
        try {
            Runner.run(
                    optional(dsl).map(File::toPath).orElseThrow(),
                    optional(templates).map(File::toPath),
                    optional(output).map(File::toPath),
                    getParserType(parser),
                    new MavenLogger(getLog())
            );
        } catch (Exception e) {
            throw new MojoExecutionException(e.getMessage(), e);
        }
    }

    private Optional<Runner.ParserType> getParserType(final String type) throws ValidationException {
        if (type == null) {
            return Optional.empty();
        }
        try {
            return Optional.of(Runner.ParserType.valueOf(type));
        } catch (IllegalArgumentException e) {
            throw new ValidationException(e);
        }
    }

    private <T> Optional<T> optional(T arg) {
        return arg == null ? Optional.empty() : Optional.of(arg);
    }
}
