// Copyright 2025 "Johan Maasing" <johan@zoom.nu>
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
grammar Common;
generatorconfig         : '{' (configitem)? (',' configitem)* '}';
configitem              : configkey ':' configvalue ;
configkey               : IDENTIFIER ;
configvalue             : (IDENTIFIER|VALUE) ;
namedTypeDeclaration    : typeName typeDeclaration ;
typeName                : IDENTIFIER ;
typeDeclaration         : '(' typeField (',' typeField)* ')' ;
typeField               : fieldName COLON fieldType ;
fieldName               : IDENTIFIER ;
fieldType               : IDENTIFIER ;

fragment LOWERCASE      : [a-z] ;
fragment UPPERCASE      : [A-Z] ;
fragment GENERICS       : '['|']'|'<'|'>' ;
fragment DOT            : '.' ;
fragment COMMENT_BEGIN  : '/*' ;
fragment COMMENT_END    : '*/' ;
fragment DIGIT          : [0-9] ;

WS                      : [ \t\n\r]+ -> skip;
COMMENT                 : COMMENT_BEGIN .*? COMMENT_END -> skip;
LEFT_ARROW              : '<-' ;
RIGHT_ARROW             : '->' ;
IDENTIFIER              : (LOWERCASE | UPPERCASE) (LOWERCASE | UPPERCASE | DIGIT | GENERICS | DOT)* ;
VALUE                   : ~[ ,{}:()/="#';*\n\r\t]+  ;
SLASH                   : '/' ;
COLON                   : ':' ;